import { Logger } from 'vite';
import { FSRouter } from './FSRouter';

export interface PageMeta {
    pageFileName: string;
    depth: number;
    group?: string;
    dirName: string;
    pathName: string;
}
export type FileType = 'page' | 'layout' | 'loader' | 'action' | 'error' | 'loading' | 'not-found';
export declare class FSNode {
    readonly tree: FSTree;
    id: number;
    /**
     * 分组
     */
    group?: string;
    /**
     * 深度，起始为 0
     */
    depth: number;
    /**
     * 【唯一】所在目录，相对于 cwd
     */
    dirName: string;
    /**
     * 路径名
     */
    pathName: string;
    /**
     * 页面，未声明页面时为空字符串
     * @ref https://reactrouter.com/en/main/route/route#elementcomponent
     */
    pageFileName: string;
    /**
     * https://reactrouter.com/en/main/start/concepts#layout-routes
     */
    layoutFileName?: string;
    /**
     * @ref https://reactrouter.com/en/main/route/loader
     */
    loaderFileName?: string;
    /**
     * @ref https://reactrouter.com/en/main/route/action
     */
    actionFileName?: string;
    /**
     * @ref https://reactrouter.com/en/main/route/error-element
     */
    errorFileName?: string;
    /**
     * @ref https://react.dev/reference/react/Suspense
     */
    loadingFileName?: string;
    /**
     * @ref https://reactrouter.com/en/main/start/faq#how-do-i-add-a-no-match-404-route-in-react-router-v6
     */
    notFoundFileName?: string;
    children: FSNode[];
    parent?: FSNode;
    constructor(tree: FSTree, pageMeta: PageMeta);
    update(meta: PageMeta): void;
    get path(): string;
    get pageFile(): string;
    private _resolveFileName;
    private _resolveFile;
    get layoutFile(): string | undefined;
    get actionFile(): string | undefined;
    get loaderFile(): string | undefined;
    get errorFile(): string | undefined;
    get loadingFile(): string | undefined;
    get notFoundFile(): string | undefined;
    private _linking;
    static parse(pageFile: string): PageMeta;
    static create(tree: FSTree, meta: PageMeta): FSNode;
}
export interface FSTreeOptions {
    cwd: string;
    logger: Logger;
    resolveFileName: (fsNode: FSNode, fileType: FileType) => string | undefined;
    resolveDuplicate: (fsNode: FSNode, duplicatePages: string[]) => number | undefined;
}
export declare class FSTree {
    readonly options: FSTreeOptions;
    root?: FSNode;
    dirMap: Map<string, FSNode>;
    fsRouter: FSRouter;
    constructor(options: FSTreeOptions);
    get cwd(): string;
    get logger(): Logger;
    mount(pages: string[]): void;
    render(tabSize?: number): string;
    private _id;
    nextId(): number;
    static makeLayoutComponentName(id: number): string;
    static makePageComponentName(id: number): string;
}
