var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
import { createUnplugin } from "unplugin";
import nodeFS from "node:fs";
import nodePath from "node:path";
import { glob } from "glob";
import { createLogger, normalizePath } from "vite";
import { assignWith, isObject, isArray, isNumber } from "lodash-es";
const pkgName = "unplugin-react-pages";
const pkgVersion = "0.1.2";
function withDefaults(defaults, inputs) {
  return assignWith(defaults, inputs, (v1, v2) => {
    if (isObject(v1)) {
      return withDefaults(v1, v2);
    }
    return v2 === void 0 ? v1 : v2;
  });
}
function indentLine(line, size = 0) {
  return " ".repeat(size) + line;
}
function jsxLikeStringify(jsxLike) {
  const stringify = (jsx) => {
    const arg1 = jsx.props ? `{${Object.entries(jsx.props).map(([key, value]) => `${key}: ${value}`).join(", ")}}` : "null";
    const arg2 = jsx.children ? `, ${jsx.children.map(stringify).join(", ")}` : "";
    return `createElement(${jsx.tag}, ${arg1}${arg2})`;
  };
  return stringify(jsxLike);
}
function ensureArray(el) {
  return isArray(el) ? el : el ? [el] : [];
}
class PageRoute {
  constructor(fsNode) {
    __publicField(this, "parent");
    this.fsNode = fsNode;
  }
  get id() {
    return this.fsNode.id;
  }
  get path() {
    const paths = [this.fsNode.path];
    const find = (fsNode) => {
      if (!fsNode) return;
      if (fsNode.group && fsNode.depth === 1) {
        paths.unshift(fsNode.path);
        return;
      }
      if (!fsNode.group && fsNode.layoutFileName) {
        return;
      }
      paths.unshift(fsNode.path);
      find(fsNode.parent);
    };
    find(this.fsNode.parent);
    return paths.length === 0 || paths.length === 1 && paths.at(0) === "" ? "" : nodePath.join(...paths);
  }
  render(indent = 0, tabSize = 4) {
    const indent_1 = indent * tabSize;
    const indent_2 = (indent + 1) * tabSize;
    const lines = [
      //
      indentLine("{", indent_1)
    ];
    lines.push(indentLine(`path: ${JSON.stringify(this.path)},`, indent_2));
    const pageName = FSRouter.makePageComponentName(this.fsNode.id);
    const element = jsxLikeStringify({
      tag: "Suspense",
      props: {},
      children: [{ tag: pageName }]
    });
    lines.push(indentLine(`element: ${element},`, indent_2));
    lines.push(indentLine("},", indent_1));
    return lines;
  }
}
class LayoutRoute {
  constructor(fsNode) {
    __publicField(this, "parent");
    __publicField(this, "children", []);
    this.fsNode = fsNode;
  }
  get id() {
    return this.fsNode.id;
  }
  render(indent = 0, tabSize = 4) {
    const indent_1 = indent * tabSize;
    const indent_2 = (indent + 1) * tabSize;
    const lines = [
      //
      indentLine("{", indent_1)
    ];
    const layoutName = FSRouter.makeLayoutComponentName(this.fsNode.id);
    const element = jsxLikeStringify({
      tag: "Suspense",
      props: {},
      children: [{ tag: layoutName }]
    });
    lines.push(indentLine(`element: ${element},`, indent_2));
    if (this.children.length) {
      lines.push(indentLine("children: [", indent_2));
      lines.push(...this.children.flatMap((child) => child.render(indent + 2, tabSize)));
      lines.push(indentLine("],", indent_2));
    }
    lines.push(indentLine("},", indent_1));
    return lines;
  }
}
class FSRouter {
  constructor(fsTree) {
    __publicField(this, "pageRoutes", /* @__PURE__ */ new Map());
    __publicField(this, "layoutRoutes", /* @__PURE__ */ new Map());
    this.fsTree = fsTree;
  }
  addPageNode(fsNode) {
    this.pageRoutes.set(fsNode.id, new PageRoute(fsNode));
  }
  addLayoutNode(fsNode) {
    this.layoutRoutes.set(fsNode.id, new LayoutRoute(fsNode));
  }
  build() {
    this.pageRoutes.forEach((pageRoute) => {
      const find = (node) => {
        if (node.layoutFileName) {
          const layoutRoute = this.layoutRoutes.get(node.id);
          if (layoutRoute) {
            pageRoute.parent = layoutRoute;
            layoutRoute.children.push(pageRoute);
            return;
          }
        }
        if (node.parent) find(node.parent);
      };
      find(pageRoute.fsNode);
    });
    this.layoutRoutes.forEach((layoutRoute) => {
      const find = (node) => {
        const parent = node.parent;
        if (!parent) return;
        if (node.group && node.depth === 1) return;
        if (parent.layoutFileName) {
          const parentRoute = this.layoutRoutes.get(parent.id);
          if (parentRoute) {
            layoutRoute.parent = parentRoute;
            parentRoute.children.push(layoutRoute);
            return;
          }
        }
        find(parent);
      };
      find(layoutRoute.fsNode);
    });
  }
  render(tabSize = 4) {
    const importPages = ["// pages"];
    const importLayouts = ["// layouts"];
    const pageLines = [];
    this.pageRoutes.forEach((route) => {
      const importName = FSRouter.makePageComponentName(route.id);
      importPages.push(`const ${importName} = lazy(() => import("${route.fsNode.pageFile}"));`);
      if (!route.parent) {
        pageLines.push(...route.render(0, tabSize));
      }
    });
    const layoutLines = [];
    this.layoutRoutes.forEach((route) => {
      const importName = FSRouter.makeLayoutComponentName(route.id);
      importLayouts.push(`const ${importName} = lazy(() => import("${route.fsNode.layoutFile}"));`);
      if (!route.parent) {
        layoutLines.push(...route.render(0, tabSize));
      }
    });
    return [
      "/**",
      ` * generated by ${pkgName}@${pkgVersion}`,
      " * @ref https://github.com/FrontEndDev-org/unplugin-react-pages",
      " * @ref https://react.dev",
      " * @ref https://reactrouter.com",
      " */",
      "",
      'import { Suspense, lazy, createElement } from "react";',
      'import { Outlet } from "react-router-dom";',
      "",
      ...importPages,
      "",
      ...importLayouts,
      "",
      "export const routes = [",
      ...pageLines,
      ...layoutLines,
      "];",
      ""
    ].join("\n");
  }
  // TODO
  update() {
  }
  static makeLayoutComponentName(id) {
    return `Layout_${id}`;
  }
  static makePageComponentName(id) {
    return `Page_${id}`;
  }
}
class FSNode {
  constructor(tree, pageMeta) {
    __publicField(this, "id");
    /**
     * 分组
     */
    __publicField(this, "group");
    /**
     * 深度，起始为 0
     */
    __publicField(this, "depth");
    /**
     * 【唯一】所在目录，相对于 cwd
     */
    __publicField(this, "dirName");
    /**
     * 路径名
     */
    __publicField(this, "pathName");
    /**
     * 页面，未声明页面时为空字符串
     * @ref https://reactrouter.com/en/main/route/route#elementcomponent
     */
    __publicField(this, "pageFileName");
    /**
     * https://reactrouter.com/en/main/start/concepts#layout-routes
     */
    __publicField(this, "layoutFileName");
    /**
     * @ref https://reactrouter.com/en/main/route/loader
     */
    __publicField(this, "loaderFileName");
    /**
     * @ref https://reactrouter.com/en/main/route/action
     */
    __publicField(this, "actionFileName");
    /**
     * @ref https://reactrouter.com/en/main/route/error-element
     */
    __publicField(this, "errorFileName");
    /**
     * @ref https://react.dev/reference/react/Suspense
     */
    __publicField(this, "loadingFileName");
    /**
     * @ref https://reactrouter.com/en/main/start/faq#how-do-i-add-a-no-match-404-route-in-react-router-v6
     */
    __publicField(this, "notFoundFileName");
    __publicField(this, "children", []);
    __publicField(this, "parent");
    this.tree = tree;
    const { pageFileName, depth, group, dirName, pathName } = pageMeta;
    this.id = this.tree.nextId();
    this.pageFileName = pageFileName;
    this.depth = depth;
    this.group = group;
    this.dirName = dirName;
    this.pathName = pathName;
    this.layoutFileName = this._resolveFileName("layout");
    this.loaderFileName = this._resolveFileName("loader");
    this.actionFileName = this._resolveFileName("action");
    this.errorFileName = this._resolveFileName("error");
    this.loadingFileName = this._resolveFileName("loading");
    this.notFoundFileName = this._resolveFileName("not-found");
    this.tree.dirMap.set(dirName, this);
    if (this.pageFileName) {
      this.tree.fsRouter.addPageNode(this);
    }
    if (this.layoutFileName) {
      this.tree.fsRouter.addLayoutNode(this);
    }
    if (depth === 0) {
      this.tree.root = this;
    }
    this._linking();
  }
  update(meta) {
    Object.assign(this, meta);
  }
  get path() {
    return this.pathName === "." ? "" : this.pathName.startsWith("[") && this.pathName.endsWith("]") ? `:${this.pathName.slice(1, -1)}` : this.pathName.startsWith("(") && this.pathName.endsWith(")") ? this.pathName.slice(1, -1) : this.pathName;
  }
  get pageFile() {
    return nodePath.join(this.tree.cwd, this.dirName, this.pageFileName);
  }
  _resolveFileName(fileType) {
    return this.tree.options.resolveFileName(this, fileType);
  }
  _resolveFile(fileName) {
    return fileName && nodePath.join(this.tree.cwd, this.dirName, fileName);
  }
  get layoutFile() {
    return this._resolveFile(this.layoutFileName);
  }
  get actionFile() {
    return this._resolveFile(this.actionFileName);
  }
  get loaderFile() {
    return this._resolveFile(this.loaderFileName);
  }
  get errorFile() {
    return this._resolveFile(this.errorFileName);
  }
  get loadingFile() {
    return this._resolveFile(this.loadingFileName);
  }
  get notFoundFile() {
    return this._resolveFile(this.notFoundFileName);
  }
  _linking() {
    const { depth, dirName: dir } = this;
    const parentDepth = depth - 1;
    const parentDir = nodePath.dirname(dir);
    const parentName = nodePath.basename(parentDir);
    const parentGroup = depth > 0 ? this.group : void 0;
    if (depth < 1) return;
    const parent = this.tree.dirMap.get(parentDir);
    if (parent) {
      this.parent = parent;
      parent.children.push(this);
    } else {
      const newParent = new FSNode(this.tree, {
        pageFileName: "",
        depth: parentDepth,
        dirName: parentDir,
        pathName: parentName,
        group: parentGroup
      });
      this.parent = newParent;
      newParent.children.push(this);
    }
  }
  static parse(pageFile) {
    const segments = pageFile.split("/");
    const pageFileName = segments.pop();
    const depth = segments.length;
    const pathName = segments.at(-1) || ".";
    const dirName = segments.join("/") || ".";
    const firstSeg = segments.at(0);
    const group = depth > 0 && (firstSeg == null ? void 0 : firstSeg.startsWith("(")) && (firstSeg == null ? void 0 : firstSeg.endsWith(")")) ? firstSeg == null ? void 0 : firstSeg.slice(1, -1) : void 0;
    return { pageFileName, depth, dirName, pathName, group };
  }
  static create(tree, meta) {
    return new FSNode(tree, meta);
  }
}
class FSTree {
  constructor(options) {
    __publicField(this, "root");
    __publicField(this, "dirMap", /* @__PURE__ */ new Map());
    __publicField(this, "fsRouter", new FSRouter(this));
    __publicField(this, "_id", 0);
    this.options = options;
  }
  get cwd() {
    return this.options.cwd;
  }
  get logger() {
    return this.options.logger;
  }
  mount(pages) {
    if (this.dirMap.size) {
      this.logger.info(`prepare to mount pages...`);
      this.root = void 0;
      this.dirMap.clear();
      this.fsRouter = new FSRouter(this);
    }
    this.logger.info(`mount pages: ${JSON.stringify(pages)}`);
    pages.forEach((page) => {
      const meta = FSNode.parse(page);
      const fsNode = this.dirMap.get(meta.dirName);
      if (fsNode) {
        if (fsNode.pageFileName) {
          this.logger.info(`fsNode page already exists [${fsNode.pageFileName}, ${meta.pageFileName}]`);
          const chooseIndex = this.options.resolveDuplicate(fsNode, [fsNode.pageFileName, meta.pageFileName]);
          this.logger.info(`choose index: ${chooseIndex}`);
          if (isNumber(chooseIndex) && chooseIndex > -1) {
            fsNode.update(meta);
          }
        } else {
          fsNode.update(meta);
        }
      } else {
        FSNode.create(this, meta);
      }
    });
    this.logger.info(`build router`);
    this.fsRouter.build();
  }
  // // TODO
  // addFileByType(fileType: FileType, fileName: string) {
  //     this.logger.info(`add file type: ${fileType} -> ${fileName}`);
  // }
  // // TODO
  // removeFileByType(fileType: FileType, fileName: string) {
  //     this.logger.info(`remove file type: ${fileType}`);
  // }
  render(tabSize = 4) {
    return this.fsRouter.render(tabSize);
  }
  nextId() {
    return this._id++;
  }
  static makeLayoutComponentName(id) {
    return `Layout_${id}`;
  }
  static makePageComponentName(id) {
    return `Page_${id}`;
  }
}
class ReactPages {
  constructor(cwd, options) {
    __publicField(this, "logger");
    __publicField(this, "fsTree");
    __publicField(this, "absPagesDir");
    __publicField(this, "devServer");
    __publicField(this, "onChange");
    __publicField(this, "_isSetup", false);
    this.cwd = cwd;
    this.options = options;
    this.logger = createLogger(options.logLevel, { prefix: `[${pkgName}]` });
    this.logger.info(`new Options: ${JSON.stringify(options)}`);
    this.absPagesDir = nodePath.join(this.cwd, options.pagesDir);
    this.fsTree = new FSTree({
      cwd: nodePath.join("/", options.pagesDir),
      logger: this.logger,
      resolveFileName: (page, fileType) => {
        const fileNames = ensureArray(this.options.fileNames[fileType]);
        if (fileNames.length === 0) return;
        return fileNames.find((fileName) => {
          const absFile = nodePath.join(this.absPagesDir, page.dirName, fileName);
          return nodeFS.existsSync(absFile) && nodeFS.statSync(absFile).isFile();
        });
      },
      resolveDuplicate: (fsNode, duplicatePages) => {
        const fileNames = ensureArray(this.options.fileNames.page);
        if (fileNames.length === 0) return;
        for (let i = 0; i < fileNames.length; i++) {
          if (duplicatePages.includes(fileNames[i])) return i;
        }
      }
    });
  }
  async generate() {
    if (!this.isSetup) {
      this.logger.info(`is not setup, try to setup again`);
      await this.setup();
    }
    const output = this.fsTree.render();
    this.logger.info(`Generated: ${output}`);
    return output;
  }
  _inPagesDir(path) {
    return path.startsWith(this.absPagesDir);
  }
  async watch(devServer, onChange) {
    this.devServer = devServer;
    this.onChange = onChange;
    this.logger.info(`setup`);
    devServer.watcher.on("unlink", (path) => {
      if (!this._inPagesDir(path)) return;
      void this.removeFile(path);
    });
    devServer.watcher.on("add", (path) => {
      if (!this._inPagesDir(path)) return;
      void this.addFile(path);
    });
    devServer.watcher.on("change", (path) => {
      if (!this._inPagesDir(path)) return;
      void this.updateFile(path);
    });
    await this.setup();
  }
  get isSetup() {
    return this._isSetup;
  }
  async setup() {
    if (this.isSetup) {
      this.logger.error(`is already setup`);
      return;
    }
    this.logger.info(`setup react pages ${this.absPagesDir}`);
    const pageFileNames = ensureArray(this.options.fileNames.page);
    if (pageFileNames.length > 0) {
      const pageFiles = await glob(
        nodePath.join("**", pageFileNames.length > 1 ? `{${pageFileNames.join(",")}}` : pageFileNames[0]),
        {
          cwd: this.absPagesDir,
          nodir: true,
          dot: false,
          ignore: this.options.excludes
        }
      );
      const pageFiles2 = pageFiles.map(normalizePath);
      this.logger.info(`Page files: ${JSON.stringify(pageFiles2)}`);
      this.fsTree.mount(pageFiles2);
    } else {
      this.logger.error(`No page file name specified`);
    }
  }
  teardown() {
    this.logger.info(`teardown`);
    this._isSetup = false;
  }
  async addFile(file) {
    var _a;
    this.logger.info(`Add file: ${file}`);
    await this.setup();
    (_a = this.onChange) == null ? void 0 : _a.call(this);
  }
  updateFile(file) {
    this.logger.info(`Update file: ${file}`);
  }
  async removeFile(file) {
    var _a;
    this.logger.info(`Remove file: ${file}`);
    await this.setup();
    (_a = this.onChange) == null ? void 0 : _a.call(this);
  }
}
const virtualModuleId = "virtual:react-pages";
const resolvedVirtualModuleId = `\0${virtualModuleId}`;
const defaultPluginOptions = () => ({
  debug: false,
  pagesDir: "src/pages",
  logLevel: "error",
  disableLazy: false,
  caseSensitive: false,
  fileNames: {
    page: ["page.jsx", "page.tsx"],
    action: ["action.jsx", "action.tsx"],
    layout: ["layout.jsx", "layout.tsx"],
    error: ["error.jsx", "error.tsx"],
    loading: ["loading.jsx", "loading.tsx"],
    loader: ["loader.jsx", "loader.tsx"],
    "not-found": ["not-found.jsx", "not-found.tsx"]
  },
  excludes: ["**/node_modules/**"]
});
const reactPages = createUnplugin((options) => {
  let reactPages2 = null;
  return {
    name: pkgName,
    enforce: "pre",
    buildStart() {
      reactPages2 = new ReactPages(process.cwd(), withDefaults(defaultPluginOptions(), options));
    },
    buildEnd() {
      reactPages2 == null ? void 0 : reactPages2.teardown();
    },
    resolveId(id) {
      if (id !== virtualModuleId) return;
      return resolvedVirtualModuleId;
    },
    loadInclude(id) {
      return id === resolvedVirtualModuleId;
    },
    async load(id) {
      if (id !== resolvedVirtualModuleId) return;
      return reactPages2 == null ? void 0 : reactPages2.generate();
    }
  };
});
export {
  pkgName,
  pkgVersion,
  reactPages
};
//# sourceMappingURL=index.mjs.map
